@@ -6,7 +6,7 @@
 ;; Homepage: https://github.com/ahyatt/triples
 ;; Package-Requires: ((seq "2.0") (emacs "28.1"))
 ;; Keywords: triples, kg, data, sqlite
-;; Version: 0.3.3
+;; Version: 0.3.4
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU General Public License as
 ;; published by the Free Software Foundation; either version 2 of the
@@ -54,6 +54,33 @@ It is invoked to make backups.")
   "The default filename triples database. If no database is
 specified, this file is used.")
 
+(defun triples-rebuild-builtin-database (db)
+  "Rebuild the builtin database DB.
+This is used in upgrades and when problems are detected."
+  (triples-with-transaction
+      db
+      (sqlite-execute db "ALTER TABLE triples RENAME TO triples_old")
+      (triples-setup-table-for-builtin db)
+      (sqlite-execute db "INSERT INTO triples (subject, predicate, object, properties) SELECT DISTINCT subject, predicate, object, properties FROM triples_old")
+      (sqlite-execute db "DROP TABLE triples_old")))
+
+(defun triples-maybe-upgrade-to-builtin (db)
+  "Check to see if DB needs to be upgraded from emacsql to builtin."
+  ;; Check to see if this was previously an emacsql database, and if so,
+  ;; change the property column to be standard for builtin sqlite.
+  (when (> (caar (sqlite-select db "SELECT COUNT(*) FROM triples WHERE properties = '(:t t)'"))
+           0)
+    (if (> (caar (sqlite-select db "SELECT COUNT(*) FROM triples WHERE properties = '()'"))
+            0)
+        (progn
+          (message "triples: detected data written with both builtin and emacsql, upgrading and removing duplicates")
+          ;; Where we can, let's just upgrade the old data.  However, sometimes we cannot due to duplicates.
+          (sqlite-execute db "UPDATE OR IGNORE triples SET properties = '()' WHERE properties = '(:t t)'")
+          ;; Remove any duplicates that we cannot upgrade.
+          (sqlite-execute db "DELETE FROM triples WHERE properties = '(:t t)'"))
+      (message "triples: detected previously used emacsql database, converting to builtin sqlite")
+      (sqlite-execute db "UPDATE triples SET properties = '()' WHERE properties = '(:t t)'"))))
+
 (defun triples-connect (&optional file)
   "Connect to the database FILE and make sure it is populated.
 If FILE is nil, use `triples-default-database-filename'."
@@ -65,7 +92,14 @@ If FILE is nil, use `triples-default-database-filename'."
   (let ((file (or file triples-default-database-filename)))
     (pcase triples-sqlite-interface
       ('builtin (let* ((db (sqlite-open file)))
-                  (triples-setup-table-for-builtin db)
+                  (condition-case nil
+                      (progn
+                        (triples-setup-table-for-builtin db)
+                        (triples-maybe-upgrade-to-builtin db))
+                    (error
+                     (message "triples: failed to ensure proper database tables and indexes.  Trying an automatic fix.")
+                     (triples-rebuild-builtin-database db)
+                     (message "triples: fix completed, if this message re-occurs please file a bug report.")))
                   db))
       ('emacsql
        (require 'emacsql)
